@@ -480,7 +480,7 @@ blit_##name (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer) \
   gint height = render->height; \
   gint dst_stride = GST_ROUND_UP_4 (width * bpp); \
   gint dst_skip; \
-  gint src_stride, src_skip; \
+  gint src_skip; \
   \
   while (ass_image) { \
     if (ass_image->dst_y > height || ass_image->dst_x > width) \
@@ -496,7 +496,6 @@ blit_##name (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer) \
     \
     w = MIN (ass_image->w, width - ass_image->dst_x); \
     h = MIN (ass_image->h, height - ass_image->dst_y); \
-    src_stride = ass_image->stride; \
     src_skip = ass_image->stride - w; \
     dst_skip = dst_stride - w * bpp; \
     \
@@ -571,13 +570,13 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
   const guint8 *src;
   guint8 *dst_y, *dst_u, *dst_v;
   gint x, y, w, h;
-  gint w2, h2;
+  gint w2;
   gint width = render->width;
   gint height = render->height;
   gint src_stride;
-  gint y_offset, y_height, y_width, y_stride;
-  gint u_offset, u_height, u_width, u_stride;
-  gint v_offset, v_height, v_width, v_stride;
+  gint y_offset, y_stride;
+  gint u_offset, u_stride;
+  gint v_offset, v_stride;
 
   y_offset =
       gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420, 0, width,
@@ -589,20 +588,6 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
       gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420, 2, width,
       height);
 
-  y_height =
-      gst_video_format_get_component_height (GST_VIDEO_FORMAT_I420, 0, height);
-  u_height =
-      gst_video_format_get_component_height (GST_VIDEO_FORMAT_I420, 1, height);
-  v_height =
-      gst_video_format_get_component_height (GST_VIDEO_FORMAT_I420, 2, height);
-
-  y_width =
-      gst_video_format_get_component_width (GST_VIDEO_FORMAT_I420, 0, width);
-  u_width =
-      gst_video_format_get_component_width (GST_VIDEO_FORMAT_I420, 1, width);
-  v_width =
-      gst_video_format_get_component_width (GST_VIDEO_FORMAT_I420, 2, width);
-
   y_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 0, width);
   u_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 1, width);
   v_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 2, width);
@@ -625,7 +610,6 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
     h = MIN (ass_image->h, height - ass_image->dst_y);
 
     w2 = (w + 1) / 2;
-    h2 = (h + 1) / 2;
 
     src_stride = ass_image->stride;
 
