@@ -768,8 +768,8 @@ gst_frei0r_mixer_init (GstFrei0rMixer * self, GstFrei0rMixerClass * klass)
 }
 
 GstFrei0rPluginRegisterReturn
-gst_frei0r_mixer_register (GstPlugin * plugin, const f0r_plugin_info_t * info,
-    const GstFrei0rFuncTable * ftable)
+gst_frei0r_mixer_register (GstPlugin * plugin, const gchar * vendor,
+    const f0r_plugin_info_t * info, const GstFrei0rFuncTable * ftable)
 {
   GTypeInfo typeinfo = {
     sizeof (GstFrei0rMixerClass),
@@ -790,7 +790,10 @@ gst_frei0r_mixer_register (GstPlugin * plugin, const f0r_plugin_info_t * info,
   if (ftable->update2 == NULL)
     return GST_FREI0R_PLUGIN_REGISTER_RETURN_FAILED;
 
-  tmp = g_strdup_printf ("frei0r-mixer-%s", info->name);
+  if (vendor)
+    tmp = g_strdup_printf ("frei0r-mixer-%s-%s", vendor, info->name);
+  else
+    tmp = g_strdup_printf ("frei0r-mixer-%s", info->name);
   type_name = g_ascii_strdown (tmp, -1);
   g_free (tmp);
   g_strcanon (type_name, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-+", '-');
