@@ -114,6 +114,9 @@ struct _GstJpegParsePrivate
   /* video state */
   gint framerate_numerator;
   gint framerate_denominator;
+
+  /* tags */
+  GstTagList *tags;
 };
 
 static void gst_jpeg_parse_dispose (GObject * object);
@@ -142,7 +145,7 @@ gst_jpeg_parse_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_jpeg_parse_sink_pad_template));
   gst_element_class_set_details_simple (element_class,
       "JPEG stream parser",
-      "Codec/Parser/Video",
+      "Video/Parser",
       "Parse JPEG images into single-frame buffers",
       "Arnout Vandecappelle (Essensium/Mind) <arnout@mind.be>");
 }
@@ -286,9 +289,10 @@ gst_jpeg_parse_parse_tag_has_entropy_segment (guint8 tag)
   return FALSE;
 }
 
-/* returns image length in bytes if parsed
- * successfully, otherwise 0 if not enough data */
-static guint
+/* returns image length in bytes if parsed successfully,
+ * otherwise 0 if more data needed,
+ * if < 0 the absolute value needs to be flushed */
+static gint
 gst_jpeg_parse_get_image_length (GstJpegParse * parse)
 {
   guint size;
@@ -350,6 +354,13 @@ gst_jpeg_parse_get_image_length (GstJpegParse * parse)
       parse->priv->last_resync = FALSE;
       parse->priv->last_offset = 0;
       return (offset + 4);
+    } else if (value == 0xd8) {
+      /* Skip this frame if we found another SOI marker */
+      GST_DEBUG ("0x%08x: SOI marker before EOI, skipping", offset + 2);
+      /* clear parse state */
+      parse->priv->last_resync = FALSE;
+      parse->priv->last_offset = 0;
+      return -(offset + 2);
     }
 
     if (value >= 0xd0 && value <= 0xd7)
@@ -403,6 +414,7 @@ gst_jpeg_parse_get_image_length (GstJpegParse * parse)
       if (noffset < 0) {
         /* ignore and continue resyncing until we hit the end
          * of our data or find a sync point that looks okay */
+        offset++;
         continue;
       }
       GST_DEBUG ("found sync at 0x%x", offset + 2);
@@ -420,7 +432,7 @@ need_more_data:
   }
 }
 
-static gboolean
+static inline gboolean
 gst_jpeg_parse_sof (GstJpegParse * parse, GstByteReader * reader)
 {
   guint8 numcomps = 0;          /* Number of components in image
@@ -499,6 +511,23 @@ gst_jpeg_parse_sof (GstJpegParse * parse, GstByteReader * reader)
   return TRUE;
 }
 
+static inline gboolean
+gst_jpeg_parse_skip_marker (GstJpegParse * parse,
+    GstByteReader * reader, guint8 marker)
+{
+  guint16 size = 0;
+
+  if (!gst_byte_reader_get_uint16_be (reader, &size))
+    return FALSE;
+
+  if (!gst_byte_reader_skip (reader, size - 2))
+    return FALSE;
+
+  GST_LOG_OBJECT (parse, "unhandled marker %x skiping %u bytes", marker, size);
+
+  return TRUE;
+}
+
 static gboolean
 gst_jpeg_parse_read_header (GstJpegParse * parse, GstBuffer * buffer)
 {
@@ -532,19 +561,27 @@ gst_jpeg_parse_read_header (GstJpegParse * parse, GstBuffer * buffer)
         break;
 
       case COM:{               /* read comment and post as tag */
-        GstTagList *tags;
         const guint8 *comment = NULL;
+        gchar *comm;
+        const gchar *env_vars[] = { "GST_JPEG_TAG_ENCODING",
+          "GST_TAG_ENCODING", NULL
+        };
 
         if (!gst_byte_reader_get_uint16_be (&reader, &size))
           goto error;
         if (!gst_byte_reader_get_data (&reader, size - 2, &comment))
           goto error;
 
-        tags = gst_tag_list_new ();
-        gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE,
-            GST_TAG_COMMENT, comment, NULL);
-        gst_element_found_tags_for_pad (GST_ELEMENT_CAST (parse),
-            parse->priv->srcpad, tags);
+        comm = (gchar *) comment;
+        comm = gst_tag_freeform_string_to_utf8 (comm, size - 2, env_vars);
+
+        if (comm) {
+          if (!parse->priv->tags)
+            parse->priv->tags = gst_tag_list_new ();
+          gst_tag_list_add (parse->priv->tags, GST_TAG_MERGE_REPLACE,
+              GST_TAG_COMMENT, comm, NULL);
+          g_free (comm);
+        }
         break;
       }
 
@@ -573,11 +610,18 @@ gst_jpeg_parse_read_header (GstJpegParse * parse, GstBuffer * buffer)
           GST_BUFFER_SIZE (buf) = exif_size;
           tags = gst_tag_list_from_exif_buffer_with_tiff_header (buf);
           gst_buffer_unref (buf);
+
           if (tags) {
-            GST_INFO_OBJECT (parse, "post exif metadata");
-            gst_element_found_tags_for_pad (GST_ELEMENT_CAST (parse),
-                parse->priv->srcpad, tags);
+            GST_INFO_OBJECT (parse, "got exif metadata");
+            if (parse->priv->tags) {
+              gst_tag_list_insert (parse->priv->tags, tags,
+                  GST_TAG_MERGE_REPLACE);
+              gst_tag_list_free (tags);
+            } else {
+              parse->priv->tags = tags;
+            }
           }
+
           GST_LOG_OBJECT (parse, "parsed marker %x: '%s' %u bytes",
               marker, id_str, size - 2);
         } else if (!strcmp (id_str, "http://ns.adobe.com/xap/1.0/")) {
@@ -595,11 +639,18 @@ gst_jpeg_parse_read_header (GstJpegParse * parse, GstBuffer * buffer)
           GST_BUFFER_SIZE (buf) = xmp_size;
           tags = gst_tag_list_from_xmp_buffer (buf);
           gst_buffer_unref (buf);
+
           if (tags) {
-            GST_INFO_OBJECT (parse, "post xmp metadata");
-            gst_element_found_tags_for_pad (GST_ELEMENT_CAST (parse),
-                parse->priv->srcpad, tags);
+            GST_INFO_OBJECT (parse, "got xmp metadata");
+            if (parse->priv->tags) {
+              gst_tag_list_insert (parse->priv->tags, tags,
+                  GST_TAG_MERGE_REPLACE);
+              gst_tag_list_free (tags);
+            } else {
+              parse->priv->tags = tags;
+            }
           }
+
           GST_LOG_OBJECT (parse, "parsed marker %x: '%s' %u bytes",
               marker, id_str, size - 2);
         } else {
@@ -629,12 +680,8 @@ gst_jpeg_parse_read_header (GstJpegParse * parse, GstBuffer * buffer)
       case DHT:
       case DQT:
         /* Ignore these codes */
-        if (!gst_byte_reader_get_uint16_be (&reader, &size))
+        if (!gst_jpeg_parse_skip_marker (parse, &reader, marker))
           goto error;
-        if (!gst_byte_reader_skip (&reader, size - 2))
-          goto error;
-        GST_LOG_OBJECT (parse, "unhandled marker %x skiping %u bytes", marker,
-            size - 2);
         break;
 
       case SOF2:
@@ -666,14 +713,12 @@ gst_jpeg_parse_read_header (GstJpegParse * parse, GstBuffer * buffer)
           memmove (&data[pos], &data[pos + size],
               GST_BUFFER_SIZE (buffer) - (pos + size));
           GST_BUFFER_SIZE (buffer) -= size;
-          reader.size -= size;
+
+          if (!gst_byte_reader_set_pos (&reader, pos - size))
+            goto error;
 #else
-          if (!gst_byte_reader_get_uint16_be (&reader, &size))
+          if (!gst_jpeg_parse_skip_marker (parse, &reader, marker))
             goto error;
-          if (!gst_byte_reader_skip (&reader, size - 2))
-            goto error;
-          GST_LOG_OBJECT (parse, "unhandled marker %x skiping %u bytes", marker,
-              size - 2);
 #endif
         } else {
           GST_WARNING_OBJECT (parse, "unhandled marker %x, leaving", marker);
@@ -691,7 +736,7 @@ gst_jpeg_parse_read_header (GstJpegParse * parse, GstBuffer * buffer)
 
 error:
   GST_WARNING_OBJECT (parse,
-      "Error parsing image header (need more that %u bytes available)",
+      "Error parsing image header (need more than %u bytes available)",
       gst_byte_reader_get_remaining (&reader));
   return FALSE;
 }
@@ -777,12 +822,20 @@ gst_jpeg_parse_push_buffer (GstJpegParse * parse, guint len)
       return GST_FLOW_ERROR;
     }
 
+    if (parse->priv->tags) {
+      GST_DEBUG_OBJECT (parse, "Pushing tags");
+      gst_element_found_tags_for_pad (GST_ELEMENT_CAST (parse),
+          parse->priv->srcpad, parse->priv->tags);
+      parse->priv->tags = NULL;
+    }
+
     parse->priv->new_segment = FALSE;
     parse->priv->caps_width = parse->priv->width;
     parse->priv->caps_height = parse->priv->height;
     parse->priv->caps_framerate_numerator = parse->priv->framerate_numerator;
     parse->priv->caps_framerate_denominator =
         parse->priv->framerate_denominator;
+    parse->priv->tags = NULL;
   }
 
   GST_BUFFER_TIMESTAMP (outbuf) = parse->priv->next_ts;
@@ -811,7 +864,7 @@ static GstFlowReturn
 gst_jpeg_parse_chain (GstPad * pad, GstBuffer * buf)
 {
   GstJpegParse *parse;
-  guint len;
+  gint len;
   GstClockTime timestamp, duration;
   GstFlowReturn ret = GST_FLOW_OK;
 
@@ -830,8 +883,12 @@ gst_jpeg_parse_chain (GstPad * pad, GstBuffer * buf)
 
     /* check if we already have a EOI */
     len = gst_jpeg_parse_get_image_length (parse);
-    if (len == 0)
+    if (len == 0) {
       return GST_FLOW_OK;
+    } else if (len < 0) {
+      gst_adapter_flush (parse->priv->adapter, -len);
+      continue;
+    }
 
     GST_LOG_OBJECT (parse, "parsed image of size %d", len);
 
@@ -876,6 +933,21 @@ gst_jpeg_parse_sink_event (GstPad * pad, GstEvent * event)
       res = gst_pad_push_event (parse->priv->srcpad, event);
       parse->priv->new_segment = TRUE;
       break;
+    case GST_EVENT_TAG:{
+      if (!parse->priv->new_segment)
+        res = gst_pad_event_default (pad, event);
+      else {
+        GstTagList *taglist = NULL;
+
+        gst_event_parse_tag (event, &taglist);
+        /* Hold on to the tags till the srcpad caps are definitely set */
+        if (!parse->priv->tags)
+          parse->priv->tags = gst_tag_list_new ();
+        gst_tag_list_insert (parse->priv->tags, taglist, GST_TAG_MERGE_REPLACE);
+        gst_event_unref (event);
+      }
+      break;
+    }
     default:
       res = gst_pad_event_default (pad, event);
       break;
@@ -913,6 +985,8 @@ gst_jpeg_parse_change_state (GstElement * element, GstStateChange transition)
       parse->priv->last_offset = 0;
       parse->priv->last_entropy_len = 0;
       parse->priv->last_resync = FALSE;
+
+      parse->priv->tags = NULL;
     default:
       break;
   }
@@ -924,6 +998,8 @@ gst_jpeg_parse_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       gst_adapter_clear (parse->priv->adapter);
+      if (parse->priv->tags)
+        gst_tag_list_free (parse->priv->tags);
       break;
     default:
       break;
