@@ -2089,15 +2089,11 @@ mpegts_try_discover_packet_size (MpegTSPacketizer * packetizer)
         if (dest[i] == 0x47 && dest[i + packetsize] == 0x47 &&
             dest[i + packetsize * 2] == 0x47 &&
             dest[i + packetsize * 3] == 0x47) {
-          gchar *str;
           packetizer->know_packet_size = TRUE;
           packetizer->packet_size = packetsize;
-          str =
-              g_strdup_printf
-              ("video/mpegts, systemstream=(boolean)true, packetsize=%d",
-              packetsize);
-          packetizer->caps = gst_caps_from_string ((const gchar *) str);
-          g_free (str);
+          packetizer->caps = gst_caps_new_simple ("video/mpegts",
+              "systemstream", G_TYPE_BOOLEAN, TRUE,
+              "packetsize", G_TYPE_INT, packetsize, NULL);
           pos = i;
           break;
         }
@@ -2107,9 +2103,13 @@ mpegts_try_discover_packet_size (MpegTSPacketizer * packetizer)
   }
   GST_DEBUG ("have packetsize detected: %d of %u bytes",
       packetizer->know_packet_size, packetizer->packet_size);
-  /* flush to sync byte */
-  if (pos > 0)
+  if (pos > 0) {
+    /* flush to sync byte */
     gst_adapter_flush (packetizer->adapter, pos);
+  } else if (!packetizer->know_packet_size) {
+    /* drop invalid data and move to the next possible packets */
+    gst_adapter_flush (packetizer->adapter, MPEGTS_MAX_PACKETSIZE);
+  }
   g_free (dest);
 }
 
@@ -2192,11 +2192,28 @@ mpegts_packetizer_push_section (MpegTSPacketizer * packetizer,
 
     data += pointer;
   }
+  /* TDT and TOT sections (see ETSI EN 300 468 5.2.5)
+   *  these sections do not extend to several packets so we don't need to use the
+   *  sections filter. */
+  if (packet->pid == 0x14) {
+    table_id = data[0];
+    section->section_length = GST_READ_UINT24_BE (data) & 0x000FFF;
+    section->buffer = gst_buffer_create_sub (packet->buffer,
+        data - GST_BUFFER_DATA (packet->buffer), section->section_length + 3);
+    section->table_id = table_id;
+    section->complete = TRUE;
+    res = TRUE;
+    GST_DEBUG ("TDT section pid:%d table_id:%d section_length: %d\n",
+        packet->pid, table_id, section->section_length);
+    goto out;
+  }
+
   /* create a sub buffer from the start of the section (table_id and
    * section_length included) to the end */
   sub_buf = gst_buffer_create_sub (packet->buffer,
       data - GST_BUFFER_DATA (packet->buffer), packet->data_end - data);
 
+
   stream = packetizer->streams[packet->pid];
   if (stream == NULL) {
     stream = mpegts_packetizer_stream_new ();
