@@ -189,27 +189,29 @@ mpegtsmux_class_init (MpegTsMuxClass * klass)
           GST_TYPE_STRUCTURE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_M2TS_MODE,
-      g_param_spec_boolean ("m2ts_mode", "M2TS(192 bytes) Mode",
+      g_param_spec_boolean ("m2ts-mode", "M2TS(192 bytes) Mode",
           "Defines what packet size to use, normal TS format ie .ts(188 bytes) "
-          "or Blue-Ray disc ie .m2ts(192 bytes).", FALSE, G_PARAM_READWRITE));
+          "or Blue-Ray disc ie .m2ts(192 bytes).", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PAT_INTERVAL,
       g_param_spec_uint ("pat-interval", "PAT interval",
           "Set the interval (in ticks of the 90kHz clock) for writing out the PAT table",
-          1, G_MAXUINT, TSMUX_DEFAULT_PAT_INTERVAL, G_PARAM_READWRITE));
+          1, G_MAXUINT, TSMUX_DEFAULT_PAT_INTERVAL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PMT_INTERVAL,
       g_param_spec_uint ("pmt-interval", "PMT interval",
           "Set the interval (in ticks of the 90kHz clock) for writing out the PMT table",
-          1, G_MAXUINT, TSMUX_DEFAULT_PMT_INTERVAL, G_PARAM_READWRITE));
+          1, G_MAXUINT, TSMUX_DEFAULT_PMT_INTERVAL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
 mpegtsmux_init (MpegTsMux * mux, MpegTsMuxClass * g_class)
 {
   mux->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&mpegtsmux_src_factory), "src");
+      gst_pad_new_from_static_template (&mpegtsmux_src_factory, "src");
   gst_pad_use_fixed_caps (mux->srcpad);
   gst_element_add_pad (GST_ELEMENT (mux), mux->srcpad);
 
@@ -493,6 +495,7 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data, GstPad * pad)
   }
 
 beach:
+  gst_caps_unref (caps);
   return ret;
 }
 
@@ -709,7 +712,7 @@ mpegtsmux_collected (GstCollectPads * pads, MpegTsMux * mux)
     }
 
     tsmux_stream_add_data (best->stream, GST_BUFFER_DATA (buf),
-        GST_BUFFER_SIZE (buf), buf, pts, -1);
+        GST_BUFFER_SIZE (buf), buf, pts, -1, !delta);
     best->queued_buf = NULL;
 
     mux->is_delta = delta;
@@ -990,6 +993,7 @@ mpegtsdemux_set_header_on_caps (MpegTsMux * mux)
   gst_structure_set_value (structure, "streamheader", &array);
   gst_pad_set_caps (mux->srcpad, caps);
   g_value_unset (&array);
+  gst_caps_unref (caps);
 }
 
 static gboolean
