@@ -461,43 +461,48 @@ gst_video_detect_class_init (gpointer klass, gpointer class_data)
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MESSAGE,
       g_param_spec_boolean ("message", "Message",
           "Post statics messages",
-          DEFAULT_MESSAGE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          DEFAULT_MESSAGE,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PATTERN_WIDTH,
       g_param_spec_int ("pattern-width", "Pattern width",
           "The width of the pattern markers", 1, G_MAXINT,
-          DEFAULT_PATTERN_WIDTH, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          DEFAULT_PATTERN_WIDTH,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PATTERN_HEIGHT,
       g_param_spec_int ("pattern-height", "Pattern height",
           "The height of the pattern markers", 1, G_MAXINT,
-          DEFAULT_PATTERN_HEIGHT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          DEFAULT_PATTERN_HEIGHT,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PATTERN_COUNT,
       g_param_spec_int ("pattern-count", "Pattern count",
           "The number of pattern markers", 0, G_MAXINT,
-          DEFAULT_PATTERN_COUNT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          DEFAULT_PATTERN_COUNT,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PATTERN_DATA_COUNT,
       g_param_spec_int ("pattern-data-count", "Pattern data count",
           "The number of extra data pattern markers", 0, G_MAXINT,
-          DEFAULT_PATTERN_DATA_COUNT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          DEFAULT_PATTERN_DATA_COUNT,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PATTERN_CENTER,
       g_param_spec_double ("pattern-center", "Pattern center",
           "The center of the black/white separation (0.0 = lowest, 1.0 highest)",
           0.0, 1.0, DEFAULT_PATTERN_CENTER,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PATTERN_SENSITIVITY,
       g_param_spec_double ("pattern-sensitivity", "Pattern sensitivity",
           "The sensitivity around the center for detecting the markers "
           "(0.0 = lowest, 1.0 highest)", 0.0, 1.0, DEFAULT_PATTERN_SENSITIVITY,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_LEFT_OFFSET,
       g_param_spec_int ("left-offset", "Left Offset",
           "The offset from the left border where the pattern starts", 0,
           G_MAXINT, DEFAULT_LEFT_OFFSET,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_BOTTOM_OFFSET,
       g_param_spec_int ("bottom-offset", "Bottom Offset",
           "The offset from the bottom border where the pattern starts", 0,
           G_MAXINT, DEFAULT_BOTTOM_OFFSET,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   trans_class->set_caps = GST_DEBUG_FUNCPTR (gst_video_detect_set_caps);
   trans_class->transform_ip = GST_DEBUG_FUNCPTR (gst_video_detect_transform_ip);
