@@ -793,7 +793,7 @@ gst_dshowvideodec_chain (GstPad * pad, GstBuffer * buffer)
     goto beach;
   }
 
-  if (GST_FLOW_IS_FATAL (vdec->last_ret)) {
+  if (vdec->last_ret < GST_FLOW_UNEXPECTED) {
     GST_DEBUG_OBJECT (vdec, "last decoding iteration generated a fatal error "
         "%s", gst_flow_get_name (vdec->last_ret));
     goto beach;
