@@ -32,7 +32,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <gst/gst.h>
@@ -401,24 +401,23 @@ gst_vdp_mpeg_dec_create_frame (GstBaseVideoDecoder * base_video_decoder)
 
 static GstFlowReturn
 gst_vdp_mpeg_dec_parse_data (GstBaseVideoDecoder * base_video_decoder,
-    GstBuffer * buf, gboolean at_eos)
+    GstBuffer * buf, gboolean at_eos, GstVideoFrame * frame)
 {
   GstVdpMpegDec *mpeg_dec = GST_VDP_MPEG_DEC (base_video_decoder);
 
   GstVdpMpegFrame *mpeg_frame;
   GstFlowReturn ret = GST_FLOW_OK;
   GstBitReader b_reader = GST_BIT_READER_INIT_FROM_BUFFER (buf);
-  guint32 sync_code;
   guint8 start_code;
 
   /* skip sync_code */
-  gst_bit_reader_get_bits_uint32 (&b_reader, &sync_code, 8 * 3);
+  gst_bit_reader_skip (&b_reader, 8 * 3);
 
   /* start_code */
-  gst_bit_reader_get_bits_uint8 (&b_reader, &start_code, 8);
+  if (!gst_bit_reader_get_bits_uint8 (&b_reader, &start_code, 8))
+    return GST_FLOW_ERROR;
 
-  mpeg_frame = (GstVdpMpegFrame *)
-      gst_base_video_decoder_get_current_frame (base_video_decoder);
+  mpeg_frame = GST_VDP_MPEG_FRAME_CAST (frame);
 
   if (start_code >= MPEG_PACKET_SLICE_MIN
       && start_code <= MPEG_PACKET_SLICE_MAX) {
@@ -465,7 +464,11 @@ gst_vdp_mpeg_dec_parse_data (GstBaseVideoDecoder * base_video_decoder,
       guint8 ext_code;
 
       /* ext_code */
-      gst_bit_reader_get_bits_uint8 (&b_reader, &ext_code, 4);
+      if (!gst_bit_reader_get_bits_uint8 (&b_reader, &ext_code, 4)) {
+        ret = GST_FLOW_ERROR;
+        gst_buffer_unref (buf);
+        goto done;
+      }
 
       GST_DEBUG_OBJECT (mpeg_dec, "MPEG_PACKET_EXTENSION: %d", ext_code);
 
@@ -511,6 +514,9 @@ gst_vdp_mpeg_dec_parse_data (GstBaseVideoDecoder * base_video_decoder,
       gst_buffer_unref (buf);
   }
 
+  if (at_eos && mpeg_frame->slices)
+    ret = gst_base_video_decoder_have_frame (base_video_decoder, TRUE, NULL);
+
 done:
   mpeg_dec->prev_packet = start_code;
 
